<?php
require_once 'base/GpevinculohEntityBase.php';

/**
 * Vinculo Historico.
 *
 * @package Entity
 *
 * @author  alessandrobartels
 *
 */
class GpevinculohEntity extends GpeVinculohEntityBase
{

    const IDMULTVINC_UNICO = 'UNICO';
    const IDMULTVINC_PRIMARIO = 'PRIMARIO';
    const IDMULTVINC_SECUNDARIO = 'SECUNDARIO';
	const VINCULO_EMPREGATICIO_MENOR_APRENDIZ = '9';
	const PERCENTUAL_FGTS_MENOR_APRENDIZ = '2';

    /** @var  $movimentacoes GpemovimentacaoEntity[] */
    private $movimentacoes;

    /**
     * Retorna o historico do vinculo no mes de competencia.
     *
     * @param integer    $nrorg           Numero da Organizacao.
     * @param integer    $nr_vinculom     Numero do Vinculo.
     * @param TkDateTime $mes_competencia Data de competencia.
     *
     * @return array GpevinculohEntity
     */
    public function retornaVinculoHistoricoMesCompetencia($nrorg, $nr_vinculom, TkDateTime $mes_competencia){
        return Gpevinculoh::retornaHistoricoMesCompetencia($nrorg, $nr_vinculom, $mes_competencia);
    }


    /**
     * Retorna o ultimo historico do vinculo.
     *
     * @param integer    $nrOrg           Numero da Organizacao.
     * @param integer    $nrVinculom     Numero do Vinculo.
     * @param TkDateTime $mesCompetencia Data de competencia.
     *
     * @return array GpevinculohEntity
     *
     * @author alessandrobartels 26/08/2011
     */
    private function _retornaHistoricoAtual($nrOrg, $nrVinculom, TkDateTime $mesCompetencia){
        return Gpevinculoh::retornaHistoricoAtual($nrOrg, $nrVinculom, $mesCompetencia);
    }


    /**
     * Retorna todos os historicos do vinculo ordenados pelo mais recente.
     *
     * @param integer $nrorg       Numero da Organizacao.
     * @param integer $nr_vinculom Numero do Vinculo.
     *
     * @return GpevinculohEntity
     *
     * @author alessandrobartels 26/08/2011
     */
    public function retornaHistoricos($nrorg, $nr_vinculom){
        return Gpevinculoh::retornaHistoricos($nrorg, $nr_vinculom);
    }


    /**
     * Adiciona os itens na lista e persiste na base.
     *
     * @return void
     */
    public function adicionaPersisteHistorico(TkDateTime $mesCompetencia) {

      // $vinculoh = clone $this;
        $dtVinculo = new TkDateTime($this->getDtmescompetenc(), TkDateTime::FORMAT_DATE);
        if ($mesCompetencia->equals($dtVinculo) === false) {

            $vinculoh = new self();
            $vinculoh->copyEntity($this);
            $vinculoh->setNrvinculoh(null);
            $vinculoh->setDtmescompetenc($mesCompetencia);

            $this->addItem($vinculoh);
        }

        $this->persist();
    }


    /* Enviado para o setforms - Testar na proxima atualizacao*/
    /*
    public function copyEntity(TkEntity $from){
        $columns = $from->getDao()->getTable()->getColumns();
        foreach ($columns as $column => $value){
            $get = 'get'.(ucfirst(strtolower($column)));
            $set = 'set'.(ucfirst(strtolower($column)));
            if (method_exists($this, $set) === true) {
                $this->$set($from->$get());
            }
        }
    }
    */

        
    /**
     * Persiste data base de ferias.
     *
     * @param TkDateTime $nova_data_base Data base de ferias.
     *
     * @return void
     */
    public function atualizaDataBaseFerias(TkDateTime $nova_data_base) {
        /* @todo gerar novo historico. Ver com Lucas solucao generica. */
        $this->setDtbaseferias($nova_data_base);
       // $this->adicionaPersisteHistorico();
    }


    /**
     * Atribui numero de dependentes de imposto de renda.
     *
     * @param integer $nr_dependentes Numero de dependentes IR.
     *
     * @return void
     */
    public function atualizaNrDependentesImpostoRenda($nr_dependentes) {
        $this->setNrdependir($nr_dependentes);
        /*          $this->_adicionaPersisteHistorico(); */
    }


     /**
      * Atribui numero de dependentes de salario familia.
      *
      * @param integer $nr_dependentes Numero de dependentes SF.
      *
      * @return void
      */
    public function atualizaNrDependentesSalarioFamilia($nr_dependentes) {
        $this->setNrdependsfam($nr_dependentes);
        /*        $this->_adicionaPersisteHistorico(); */
    }


    /**
     * Inclui novo vinculo historico.
     *
     * @param integer $nrOrg                Numero da Organizacao de Trabalho.
     * @param integer $nrVinculom           Numero do vinculo mestre.
     * @param integer $nrEscalaTrabm        Numero da escala de trabalho.
     * @param integer $nrSituFuncm          Numero da situacao funcional.
     * @param string  $dtVigencia           Data de vigencia do historico.
     * @param string  $contribuicaoSindical Flag de contribuicao sindical.
     * @param string  $tpSalario            Tipo de salario do vinculo.
     * @param float   $valorSalario         Valor do salario real.
     * @param string  $idRemuneracao        Identificador do tipo de remuneracao.
     * @param string  $dtBaseFerias         Data base de ferias.
     *
     * @return void
     *
     * @author alessandrobartels 06/09/2011
     */
    public function incluiVinculoH(
        $nrOrg, $nrVinculom, $nrEscalaTrabm, $nrSituFuncm, $dtVigencia, $contribuicaoSindical,
        $dtBaseFerias, $nrTpModalidade, $extra = array()
    ){

        $vinculoH = new self();
        $vinculoH->setNrorg($nrOrg);
        $vinculoH->setNrvinculom($nrVinculom);
        $vinculoH->setDtmescompetenc($dtVigencia);

        $vinculoH->setNrescalatrabm($nrEscalaTrabm);
        // $vinculoH->setNrcargom($nrcargom);.
        $vinculoH->setNrsitufuncm($nrSituFuncm);

        $vinculoH->setNrtpmodalidsal($nrTpModalidade);
        $vinculoH->setIdcontribuisind($contribuicaoSindical);
        $vinculoH->setDtbaseferias($dtBaseFerias);
        //$vinculoH->setNrestrutsind($extra['']);


        foreach ($extra as $dado_nome => $dado_valor) {
            $nome_metodo_get = 'get'.(ucfirst(strtolower($dado_nome)));
            $nome_metodo_set = 'set'.(ucfirst(strtolower($dado_nome)));
            if (method_exists($vinculoH, $nome_metodo_set) === true) {
                if (!$vinculoH->$nome_metodo_get())
                    $vinculoH->$nome_metodo_set($dado_valor);
//                  $dado_valor = self::trataValorDePessoa($dado_nome, $dado_valor);
            }
        }


        $vinculoH->persist();

        unset($vinculoH);
    }


    /**
     * Insere um historico com data de competencia projetada.
     *
     * @param TkDateTime        $mes_competencia_projetado Mes de competencia projetado.
     *
     * @return GpevinculohEntity
     */
    public function projetaVinculo(TkDateTime $mes_competencia_projetado) {
	
        $mes_competencia_atual = new TkDateTime($this->getDtmescompetenc(), TkDateTime::FORMAT_DATE);	
        if ($mes_competencia_projetado->equals($mes_competencia_atual) === true) {
            return $this;
        } else {
            // Pesquisa o vinculo no mes de competencia.
            $vinculo = $this->retornaVinculoHistoricoMesCompetencia(
                $this->getNrorg(), $this->getNrvinculom(),
                $mes_competencia_projetado->toString(TkDateTime::FORMAT_DATE)
            );

            if ($vinculo === null) {
                $vinculo_aux = new self();

                // Copia o objeto em uma nova instancia.
                $vinculo_aux->copy($this);
                $vinculo_aux->setDtmescompetenc($mes_competencia_projetado);
                $vinculo_aux->setNrorg($this->getNrorg());

                /*$vinculo_aux->setNrvinculom($this->getNrvinculom());
                  $vinculo_aux->setIdtpsalariovinc($this->getIdtpsalariovinc());
                  $vinculo_aux->setIdcontribuisind($this->getIdcontribuisind());
                  $vinculo_aux->setDtinclusao(now());

                 */
                // $this->addItem($vinculo_aux, $vinculo_aux->getNrvinculoh());.
                $this->addItem($vinculo_aux);

                return $vinculo_aux;
            } else {
                return $vinculo;
            }
        }
    }


    /**
     * Retorna posicao do historico do vinculo.
     *
     * @param integer    $nrOrg          Numero da Org.
     * @param integer    $nrVinculom     Numero do Vinculo.
     * @param TkDateTime $mesCompetencia Data de competencia.
     *
     * @return integer | boolean
     *
     * @author alessandrobartels 26/08/2011
     */
    private function _procuraVinculoH($nrOrg, TkDateTime $mesCompetencia, $nrVinculom){

        $itens = $this->getItems();
        if (empty($itens) === true) {
            // $this->_inicializaLista($vinculoh->getNrorg(), $vinculoh->getNrvinculom(), $dtOcorrencia);.
            $this->_inicializaLista($nrOrg, $nrVinculom, $mesCompetencia);
            $itens = $this->getItems();
        }

        foreach ($itens as $posicao => $item) {
            if ($mesCompetencia->equals($item->getDtmescompetenc(), TkDateTime::FORMAT_DATE) === true) {
                return $posicao;
            }
        }

        return;

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrg      Numero da Organizacao.
     * @param integer $nrVinculom Numero do Vinculo.
     *
     * @return void
     */
    public function carregaVinculosHistoricos($nrOrg, $nrVinculom){
        $itens = $this->getItems();

        if (count($itens) === 0) {
            $objVinculoH = $this->retornaHistoricos($nrOrg, $nrVinculom);
            foreach ($objVinculoH as $item) {
                $this->addItem($item);
            }
        }

    }


    /**
     * Enter description here ...
     *
     * @return void
     */
    public function apagaVinculosHistoricos(){
        $itens = $this->getItems();
        foreach ($itens as $item) {
            $this->removeItem($item);
        }
    }


    /**
     * Adiciona itens de GpevinculohEntity na lista.
     *
     * @param integer    $nrOrg          Numero da Organizacao de Trabalho.
     * @param integer    $nrVinculom     Numero do vinculo.
     * @param TkDateTime $mesCompetencia Data de competencia.
     *
     * @return void
     *
     * @author alessandrobartels - 26/08/2011
     */
    private function _inicializaLista($nrOrg, $nrVinculom, TkDateTime $mesCompetencia){
        $vinculos = $this->_retornaHistoricoAtual($nrOrg, $nrVinculom, $mesCompetencia);
        foreach ($vinculos as $vinculo) {
            $this->addItem($vinculo);
        }
    }


    /**
     * Altera salario do Vinculo.
     *
     * @param TkDateTime $mes_competencia Mes de competencia.
     * @param float      $salario         Valor do salario.
     *
     * @return void
     *
     * @author alessandrobartels
     *         alessandrobartels 30/08/2011 - Foi retirado a parametrizacao da propria instancia. Corrigido as chamadas
     *                                        dentro do metodo ($this).
     */
    public function alteraSalarioVinculo(TkDateTime $mes_competencia, $salario){

        // Retorna lista.
        $vinculos = $this->getItems();
        // Se lista vazia, popula lista.
        if (empty($vinculos) === true) {
            // $this->_inicializaLista($vinculoh->getNrorg(), $vinculoh->getNrvinculom(), $dtOcorrencia);.
            $this->_inicializaLista($this->getNrorg(), $this->getNrvinculom(), $mes_competencia);
        }

        // Procura vinculo na lista.
        $posicao = $this->_procuraVinculoH($this->getNrvinculom(), $mes_competencia);
        if ($posicao !== false) {
            $vinculo = $this->getItem($posicao);
        } else {
            $vinculo = null;
        }

        // Se vinculo nao existe cria um novo.
        if ($vinculo !== null) {
            $vinculo->setVrsalariovinc(round($salario, 2));
            $vinculo->setDtultatu(now());
        } else {
            $vinculo = new self();

            $vinculo->copy($this);
            $vinculo->setVrsalariovinc(round($salario, 2));
            $vinculo->setDtinclusao(now());
            $vinculo->setDtultatu(now());
            $vinculo->setDtmescompetenc($mes_competencia);
            $vinculo->setNrvinculoh(rand(1, 10000));
            // $this->addItem($vinculo, $vinculo->getNrvinculoh());.
            $this->addItem($vinculo);
        }
    }


    /**
     * Converte o valor do salario para valor mensal.
     *
     * @param string $ambiente_salario_operador Ambiente de Salario do Operador.
     * @param float  $jornada_diaria            Jornada diaria.
     *
     * @return void
     *
     * @author  alessandrobartels
     * @version 18/04/2011
     */
    public function converteValorSalarioParaMensal($ambiente_salario_operador, $jornada_diaria){

        /*
          if ($ambiente_salario_operador === GpeparamoperEntity::IDSALARIOOPER_REAL) {
            $salario = $this->getVrsalariovinc();
            $id_tipo_salario = $this->getIdtpsalariovinc();
        } else {
            $salario = $this->getVrsalacenario();
            $id_tipo_salario = $this->getIdtpsalacenario();
        }

        switch ($id_tipo_salario){
        case self::IDTPSALARIOVINC_HORA:
            $salario = $salario * $jornada_diaria * 30; break;
        case self::IDTPSALARIOVINC_DIARIO:
            $salario = $salario * 30; break;
        case self::IDTPSALARIOVINC_QUINZENAL:
            $salario = (($salario / 15) * 30); break;
        case self::IDTPSALARIOVINC_SEMANAL:
            $salario = (($salario / 7) * 30); break;
        }

        return $salario;
        */
    }


    /**
     * Converte o valor do salario para valor hora.
     *
     * @param string $ambiente_salario_operador Ambiente de Salario do Operador.
     * @param float  $jornada_diaria            Jornada diaria.
     *
     * @return void
     *
     * @author  alessandrobartels / brunofeliciano
     * @version 17/08/2011
     */
    public function converteValorSalarioParaHora($ambiente_salario_operador, $jornada_diaria){

        /*     if ($ambiente_salario_operador === GpeparamoperEntity::IDSALARIOOPER_REAL) {
            $salario = $this->getVrsalariovinc();
            $id_tipo_salario = $this->getIdtpsalariovinc();
        } else {
            $salario = $this->getVrsalacenario();
            $id_tipo_salario = $this->getIdtpsalacenario();
        }

        switch ($id_tipo_salario){
        case self::IDTPSALARIOVINC_DIARIO:
            $salario = $salario / $jornada_diaria; break;
        case self::IDTPSALARIOVINC_QUINZENAL:
            $salario = (($salario / 15) / $jornada_diaria); break;
        case self::IDTPSALARIOVINC_SEMANAL:
            $salario = (($salario / 7) / $jornada_diaria); break;
        case self::IDTPSALARIOVINC_MENSAL:
            $salario = (($salario / 30) / $jornada_diaria); break;
        }

        return $salario;
        */
    }


    public function getUltimoVinculoHistorico($nrOrg, $mesCompetencia, $nrVinculo){
        if (count($this->getItems()) === 0 ){
            $this->_inicializaLista($nrOrg, $nrVinculo, $mesCompetencia);
        }
        return end($this->getItems());
    }


    /**
     * Retorna o vinculo historico no mes de competencia ou o mais atual
     *
     * @param integer    $nrOrg          Numero da Organizacao
     * @param TkDateTime $mesCompetencia Mes de competencia
     * @param integer    $nrVinculom     Numero do vinculo
     *
     * @return GpevinculohEntity
     *
     * @author alessandrobartels 07/01/13
     *
     */
    public function getVinculoHistorico($nrOrg, TkDateTime $mesCompetencia, $nrVinculom){

        // Procura vinculo na lista.
        $posicao = $this->_procuraVinculoH($nrOrg, $mesCompetencia, $nrVinculom);
        if (($posicao !== false) && ($posicao !== null)) {
            $vinculoh = $this->getItem($posicao);

        } else {
            $vinculo = $this->getUltimoVinculoHistorico($nrOrg, $mesCompetencia, $nrVinculom);
            $vinculoh = new self();
            $vinculoh->copy($vinculo);
        }

        return $vinculoh;
    }

    
    public function atualizaEscalaVinculoHistoricoPorAlteracao($nrOrg, TkDateTime $mesCompetencia, $nrVinculom, $nrEscala){
        $vinculoh = $this->getVinculoHistorico($nrOrg, $mesCompetencia, $nrVinculom);
        $vinculoh->setNrescalatrabm($nrEscala);
        $vinculoh->persist();
    }


    public function atualizaSituacaoFuncionalVinculoHistoricoPorAlteracao($nrOrg, TkDateTime $mesCompetencia, $nrVinculom, $nrSituacao){
        $vinculoh = $this->getVinculoHistorico($nrOrg, $mesCompetencia, $nrVinculom);
        $vinculoh->setNrsitufuncm($nrSituacao);
        $vinculoh->persist();
    }


    public function atualizaCargoVinculoHistoricoPorAlteracao(){

    }


    public function atualizaFuncaoVinculoHistoricoPorAlteracao(){

    }


    public function atualizaEstruturaLegalVinculoHistoricoPorAlteracao(){

    }

    
    public function atualizaEstruturaGerencialVinculoHistoricoPorAlteracao(){

    }

    public function setVinculoHistorico($nrorg, $nrvinculom, $dtmescompetenc, $nrescalatrabm, $nrsitufuncm,
                                           $nrtpmodalidade, $contribuicaosindical, $dtbaseferias, $nrestrutsind,
                                           $idtppagamento, $idremuneracao, $nrestrutgeren, $nrestrutlegal, $nrfuncao,
                                           $nrcargo, $nrtpmovtransfm)
    {
        $this->setNrorg($nrorg);
        $this->setNrvinculom($nrvinculom);
        $this->setDtmescompetenc($dtmescompetenc);
        $this->setNrescalatrabm($nrescalatrabm);
        $this->setNrsitufuncm($nrsitufuncm);
        $this->setNrtpmodalidsal($nrtpmodalidade);
        $this->setIdcontribuisind($contribuicaosindical);
        $this->setDtbaseferias($dtbaseferias);
        $this->setNrestrutsind($nrestrutsind);

        $this->setIdtppagamento($idtppagamento);
        $this->setIdremuneracao($idremuneracao);
        $this->setNrestrutgeren($nrestrutgeren);
        $this->setNrestrutlegal($nrestrutlegal);
        $this->setNrfuncao($nrfuncao);
        $this->setNrcargo($nrcargo);
        $this->setNrtpmovtransfm($nrtpmovtransfm);
    }


}

